<?php

namespace Drupal\entityreference_view_mode\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;

/**
 * Plugin implementation of the 'field_example_text' widget.
 *
 * @FieldWidget(
 *   id = "field_entityreference_view_mode_widget",
 *   module = "entityreference_view_mode",
 *   label = @Translation("Content plus view mode"),
 *   field_types = {
 *     "field_entityreference_view_mode"
 *   }
 * )
 */
class EntityReferenceViewModeWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $type = $this->getFieldSetting('target_type') ? $this->getFieldSetting('target_type') : 'node';
    $selected_bundles = $this->getFieldSetting('bundles');
    $selected_view_modes = $this->getFieldSetting('view_modes');
    // Remove zero values.
    $selected_view_modes = array_filter($selected_view_modes);

    // Re-instate node. prefix.
    $selected_view_modes = preg_filter('/^/', $type . '.', $selected_view_modes);

    $view_mode_classes = \Drupal::entityTypeManager()->getStorage('entity_view_mode')->loadMultiple($selected_view_modes);

    $view_modes = [];
    foreach ($view_mode_classes as $id => $class) {
      $view_modes[$id] = $class->label();
    }

    $content_default = isset($items[$delta]->content) ? \Drupal::entityTypeManager()->getStorage($type)->load($items[$delta]->content) : '';

    $element['target_type'] = [
      '#type' => 'hidden',
      '#value' => $type
    ];


    $element['content'] = [
      '#type' => 'entity_autocomplete',
      '#target_type' => $type,
      '#title' => t('Content'),
      '#title_display' => 'hidden',
      '#default_value' => $content_default,
      '#selection_handler' => 'default',
      '#selection_settings' => [
        'target_bundles' => $selected_bundles,
      ],
      '#prefix' => '<div class="container-inline">',
    ];

    $element['view_mode'] = [
      '#type' => 'select',
      '#title' => 'View Mode',
      '#title_display' => 'hidden',
      '#default_value' => isset($items[$delta]->view_mode) ? $items[$delta]->view_mode : '',
      '#options' => $view_modes,
      '#suffix' => '</div>',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function validateFormElement($element, FormStateInterface $form_state) {
    $value = $element['#value'];
  }

}
